"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deployer = void 0;
const scrapbox_client_1 = require("../scrapbox-client");
const file_loaders_1 = require("./internal/file-loaders");
const find_1 = require("./internal/find");
class Deployer {
    constructor(projectName, token) {
        this.projectName = projectName;
        this.token = token;
    }
    async deploy(config) {
        const taskName = `${this.projectName}/${config.targetPageName}/${config.codeBlockLabel}`;
        console.log(`[scrapbox-tools/deploy] task started: ${taskName}`);
        try {
            const client = await scrapbox_client_1.getScrapboxClient({ projectName: this.projectName, token: this.token });
            const sourceCode = await file_loaders_1.loadSourceCode(config.sourceFilePath);
            const page = await client.getPage(config.targetPageName);
            // NOTE: assuming code-block exists already
            const lineId = find_1.findNextLineIdOrFail(config.codeBlockLabel, page.lines);
            // FIXME: bad type assertion
            await client.changeLine(config.targetPageName, { type: 'update', id: lineId, text: sourceCode });
            console.log(`[scrapbox-tools/deploy] task completed: ${taskName}`);
        }
        catch (e) {
            console.error('[scrapbox-tools/deploy] script errored ', e);
            throw e;
        }
    }
}
exports.Deployer = Deployer;
//# sourceMappingURL=deployer.js.map