"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.retrieveResponse = exports.getScrapboxWebsocketResponseStreams = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const constants_1 = require("../constants");
const message_1 = require("./message");
const event_stream_1 = require("./websocket/event-stream");
const responseTypes = {
    error: 'error',
    initialize: 'initialize',
    response: 'response',
    others: 'others',
};
const toResponse = (event) => {
    if (typeof event.data !== 'string') {
        return {
            type: responseTypes.error,
            data: String(event.data),
        };
    }
    const [header, data] = message_1.parseMessage(event.data);
    switch (header) {
        // just after connection opened
        case constants_1.socketIoHeaders.initialize: {
            return {
                type: responseTypes.initialize,
                data: data,
            };
        }
        default: {
            // result of this.send()
            if (header.startsWith(constants_1.socketIoHeaders.receive)) {
                return {
                    type: responseTypes.response,
                    sid: header.slice(constants_1.socketIoHeaders.receive.length),
                    data: data,
                };
            }
            return {
                type: responseTypes.others,
                data: data,
            };
        }
    }
};
const isError = (data) => data.type === responseTypes.error;
const isInitialize = (data) => data.type === responseTypes.initialize;
const isResponse = (data) => data.type === responseTypes.response;
const isOthers = (data) => data.type === responseTypes.others;
const getScrapboxWebsocketResponseStreams = (socket) => {
    const { open, close, error, message } = event_stream_1.getWebsocketEventStreams(socket);
    const response = message.pipe(operators_1.map(toResponse));
    return {
        open,
        close,
        error,
        message,
        initialize: response.pipe(operators_1.filter(isInitialize)),
        response: response.pipe(operators_1.filter(isResponse)),
        others: response.pipe(operators_1.filter(isOthers)),
        errorResponse: response.pipe(operators_1.filter(isError)),
    };
};
exports.getScrapboxWebsocketResponseStreams = getScrapboxWebsocketResponseStreams;
const retrieveResponse = (response$, sid, timeoutMs = constants_1.websocketResponseTimeout) => {
    return rxjs_1.firstValueFrom(response$.pipe(operators_1.timeout(timeoutMs), operators_1.first((response) => response.sid === sid), operators_1.map((response) => response.data)));
};
exports.retrieveResponse = retrieveResponse;
//# sourceMappingURL=response.js.map