"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketClient = void 0;
const rxjs_1 = require("rxjs");
const constants_1 = require("./constants");
const message_1 = require("./internal/message");
const payload_1 = require("./internal/payload");
const response_1 = require("./internal/response");
const getter_1 = require("./internal/websocket/getter");
const util_1 = require("./internal/websocket/util");
class WebsocketClient {
    constructor(token = '', websocketGetterFunction = getter_1.getDefaultWebsocket) {
        this.token = token;
        this.websocketGetterFunction = websocketGetterFunction;
        this.response$ = rxjs_1.NEVER;
        this.open$ = rxjs_1.NEVER;
        // currently joined-room, need cache to re-join on reconnect
        this.room = null;
        // wait request until connection opened
        this.sid = 0;
        this.initialize();
    }
    commit(params) {
        return this.send(payload_1.createCommitPayload(params));
    }
    join(params) {
        this.room = params;
        return this.send(payload_1.createJoinPayload(params));
    }
    /**
     * Connect to websocket and register events.
     */
    initialize() {
        if (this.socket) {
            this.socket.close();
        }
        this.socket = this.websocketGetterFunction(getter_1.getScrapboxWebsocketAuthOptions(this.token));
        const { close, open, initialize, response } = response_1.getScrapboxWebsocketResponseStreams(this.socket);
        this.open$ = open;
        this.response$ = response;
        close.subscribe(() => this.initialize()); // retry if disconnected
        initialize.subscribe((message) => {
            this.socket.send(constants_1.socketIoHeaders.connected);
            // start ping
            rxjs_1.interval(message.data.pingInterval).subscribe(() => this.socket.send(constants_1.socketIoHeaders.ping));
            // for reconnection after closed
            if (this.room) {
                this.join(this.room);
            }
        });
    }
    async send(payload) {
        const [sid, data] = message_1.createMessage(this.getIdAndIncrement(), payload);
        if (util_1.isOpen(this.socket)) {
            this.socket.send(data);
        }
        else {
            this.open$.subscribe(() => this.socket.send(data));
        }
        return response_1.retrieveResponse(this.response$, sid);
    }
    getIdAndIncrement() {
        return String(this.sid++);
    }
}
exports.WebsocketClient = WebsocketClient;
//# sourceMappingURL=websocket-client.js.map