import { ID } from '../../common';
import { ChangeRequestCreateParams, CommitChangeRequest } from './request';
export declare const createCommitPayload: (params: {
    projectId: string;
    userId: string;
    pageId: string;
    parentId: string;
    changes: ChangeRequestCreateParams[];
}) => {
    readonly method: "commit";
    readonly data: {
        readonly kind: "page";
        readonly userId: string;
        readonly projectId: string;
        readonly pageId: string;
        readonly parentId: string;
        readonly changes: ({
            readonly _insert: string;
            readonly lines: {
                readonly id: string;
                readonly text: string;
            };
        } | {
            readonly _update: string;
            readonly lines: {
                readonly text: string;
            };
        } | {
            readonly _delete: string;
            readonly lines: -1;
        } | {
            readonly title: string;
        } | {
            readonly descriptions: readonly [string];
        })[];
        readonly cursor: null;
        readonly freeze: true;
    };
};
export declare const createJoinPayload: (params: {
    projectId: string;
    pageId: string;
}) => {
    readonly method: "room:join";
    readonly data: {
        readonly pageId: string;
        readonly projectId: string;
        readonly projectUpdatesStream: false;
    };
};
export declare type RequestPayload = ReturnType<typeof createCommitPayload> | ReturnType<typeof createJoinPayload>;
declare type Error = {
    message: string;
    name: string;
    value: string;
    kind: 'required' | 'ObjectId' | string;
    path: '_id' | 'title' | string;
    stack: string;
};
export declare type CommitError = Error & {
    properties: {
        message: string;
        type: 'required';
        path: 'title';
        value: '';
    };
};
export declare type JoinError = Error & {
    stringValue: string;
};
export declare type CommitErrorPayload = {
    error: {
        name: string;
        message: string;
        errors: {
            title?: CommitError;
            titleLc?: CommitError;
        };
    };
};
export declare type JoinErrorPayload = {
    error: {
        errors: JoinError[];
    };
};
export declare type ErrorResponsePayload = CommitErrorPayload | JoinErrorPayload;
export declare type ConnectionOpenResponsePayload = {
    sid: string;
    upgrades: [];
    pingInterval: number;
    pingTimeout: number;
};
export declare type CommitSuccessResponsePayload = {
    data: {
        commitId: string;
    };
};
export declare type CommitResponsePayload = CommitSuccessResponsePayload | ErrorResponsePayload;
export declare type JoinRoomSuccessResponsePayload = {
    data: {
        success: true;
        pageId: string;
        projectId: string;
    };
};
export declare type JoinRoomResponsePayload = JoinRoomSuccessResponsePayload | ErrorResponsePayload;
export declare type SendResponsePayload = CommitResponsePayload[] | JoinRoomResponsePayload[];
export declare type CursorResponsePayload = [
    'cursor',
    {
        user: {
            id: string;
            displayName: string;
        };
        pageId: string;
        position: {
            line: number;
            char: number;
        };
        visible: true;
        socketId: string;
    }
];
export declare type ExternalCommitData = {
    kind: 'page';
    parentId: string;
    changes: CommitChangeRequest[];
    cursor: null;
    pageId: string;
    userId: ID;
    projectId: string;
    freeze: true;
    id: string;
};
export declare type ExternalCommitResponsePayload = ['commit', ExternalCommitData];
export declare type ExternalResponsePayload = CursorResponsePayload | ExternalCommitResponsePayload;
export declare type WebsocketResponsePayload = ConnectionOpenResponsePayload | CursorResponsePayload | ExternalCommitResponsePayload | SendResponsePayload;
export {};
